use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Message {
    pub from: String,
    pub message_kind: MessageKind,
    pub length: u16,
    pub content: String,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum MessageKind {
    Test,
    Move,
    CreatePlayer,
    CreateLobby,
}

impl Message {
    pub fn new(from: &str, message_kind: MessageKind, content: impl Into<String>) -> anyhow::Result<Self> {
        let from = from.into();
        let content = content.into();
        let length = content.len() as u16;

        Ok(Self {
            from,
            message_kind,
            length,
            content,
        })
    }

    pub fn encode(&self) -> anyhow::Result<String> {
        let message_json: String = serde_json::to_string(self)?;

        Ok(message_json)
    }

    pub fn decode(buffer: &[u8]) -> anyhow::Result<Self> {
        let message: Message = serde_json::from_slice(buffer)?;

        Ok(message)
    }
}
