use tokio::{io::{AsyncReadExt, AsyncWriteExt, BufReader}, net::{tcp::WriteHalf, TcpListener}};
use std::net::SocketAddr;
use crate::{message::{Message, MessageKind}, message_read::MessageReader, server};

pub struct Server {
    pub host: String,
    pub port: u16,
}

impl Server {
    pub fn new(host: impl Into<String>, port: u16) -> Self {
        Server {
            host: host.into(),
            port,
        }
    }

    #[allow(unreachable_code)]
    pub async fn run(&mut self) -> anyhow::Result<()> {
        let listener = TcpListener::bind(format!(
            "{}:{}",
            self.host,
            self.port
        )).await?;

        println!("Server is running on {}:{}", self.host, self.port);

        loop {
            let (mut socket, addr) = listener.accept().await?;
            println!("Connection received from {}", addr);

            tokio::task::spawn(async move {
                let mut message_reader = MessageReader::new();


                let (mut rd, mut wr) = socket.split();
                let mut bufread = BufReader::new(&mut rd);

                'handler: loop {
                    let mut buffer = vec![];
                    let _bytes_read = match bufread.read_to_end(&mut buffer).await {
                        Ok(0) => {
                            println!("connection closed by client");
                            break 'handler;
                        },
                        Ok(_n) => {
                            let message = message_reader.read(&buffer).expect("failed to parse message");
                            println!("{:?}", message);                      },
                        Err(e) => {
                            eprintln!("error reading from stream: {}", e);
                            break 'handler;  
                        },
                    };
                    // if tokio::io::copy(&mut rd, &mut buffer).await.is_err() {
                    //     eprintln!("failed to copy");
                    // }
                    
                    // let message =  message_reader.read(&buffer)?;
                    // println!("{:?}", message);

                    // let local_address = &rd.local_addr()?;
                    // let message_client = Message::new(&local_address.to_string(), MessageKind::Test, "toto test")?;
                    // wr.write_all(&message_client.encode()?.as_bytes()).await?;                       
                }

                Ok::<(), anyhow::Error>(())
            });
        }

        Ok(())
    }
}
